function featureVector = gaborFeatures(img,gaborArray,d1,d2)

% GABORFEATURES extracts the Gabor features of the image.
% It creates a column vector, consisting of the image's Gabor features.
% The feature vectors are normalized to zero mean and unit variance.
%
%
% Inputs:
%       img         :	Matrix of the input image 
%       gaborArray	:	Gabor filters bank created by the function gaborFilterBank
%       d1          :	The factor of downsampling along rows.
%                       d1 must be a factor of n if n is the number of rows in img.
%       d2          :	The factor of downsampling along columns.
%                       d2 must be a factor of m if m is the number of columns in img.
%               
% Output:
%       featureVector	:   A column vector with length (m*n*u*v)/(d1*d2). 
%                           This vector is the Gabor feature vector of an 
%                           m by n image. u is the number of scales and
%                           v is the number of orientations in 'gaborArray'.
%
%
% Sample use:
% 
% img = imread('cameraman.tif');
% gaborArray = gaborFilterBank(5,8,39,39);  % Generates the Gabor filter bank
% featureVector = gaborFeatures(img,gaborArray,4,4);   % Extracts Gabor feature vector, 'featureVector', from the image, 'img'.
% 
% 
%   Details can be found in:
%   
%   M. Haghighat, S. Zonouz, M. Abdel-Mottaleb, "Identification Using 
%   Encrypted Biometrics," Computer Analysis of Images and Patterns, 
%   Springer Berlin Heidelberg, pp. 440-448, 2013.
% 
% 
% (C)	Mohammad Haghighat, University of Miami
%       haghighat@ieee.org
%       I WILL APPRECIATE IF YOU CITE OUR PAPER IN YOUR WORK.


if (nargin ~= 4)    % Check correct number of arguments
    error('Use correct number of input arguments!')
end

if size(img,3) == 3	% % Check if the input image is grayscale
    img = rgb2gray(img);
end

img = double(img);


%% Filtering

% Filter input image by each Gabor filter
[u,v] = size(gaborArray);
gaborResult = cell(u,v);
PSI = cell(u,v);
for i = 1:u
    for j = 1:v
        gaborResult{i,j} = conv2(img,gaborArray{i,j},'same');
%         % J{u,v} = filter2(G{u,v},I);

        temp=gradient(real(gaborArray{i,j}));
        temp2=gradient(temp);
        A=abs(conv2(img,temp,'same')).^2;
        B=conv2(img,real(gaborArray{i,j}),'same');
        C=conv2(img,temp2,'same');
%         
%         temp=gradient(real(gaborArray{i,j}));
%         temp2=gradient(temp);
%         A=abs(conv2(img,temp,'same')).^2;
%         B=conv2(img,real(gaborArray{i,j}),'same');
%         C=conv2(img,temp2,'same');
% % 
        PSI{i,j}=A-B.*C;
        
    end
end


%% Feature Extraction

% Extract feature vector from input image
[n,m] = size(img);
s = (n*m)/(d1*d2);
l = s*u*v;
featureVector = zeros(l,1);
c = 0;
for i = 1:u
    for j = 1:v
        
        c = c+1;
%         gaborAbs = abs(PSI{i,j});
        gaborAbs = abs(gaborResult{i,j});
        gaborAbs = downsample(gaborAbs,d1);
        gaborAbs = downsample(gaborAbs.',d2);
        gaborAbs = reshape(gaborAbs.',[],1);
        
        % Normalized to zero mean and unit variance. (if not applicable, please comment this line)
%         gaborAbs = (gaborAbs-mean(gaborAbs))/std(gaborAbs,1);
        
        featureVector(((c-1)*s+1):(c*s)) = gaborAbs;
        
    end
end

% bg=zeros(n,m,u*v);
% c=0;
% for i=1:u
%     for j=1:v
%         c=c+1;
%         bg(:,:,c)=(PSI{i,j}-mean(PSI{i,j}))/std(PSI{i,j},1);
%     end;
% end;


%% Show filtered images

% % Show real parts of Gabor-filtered images
% figure;
% for i = 1:u
%     for j = 1:v        
%         subplot(u,v,(i-1)*v+j)    
%         imshow(real(gaborResult{i,j}),[]);
%     end
% end
% % title('Real parts of Gabor filters')

% Show magnitudes of Gabor-filtered images
% c=0;
% figure;
% for i = 1:u
%     for j = 1:v        
%         c=c+1;
%         subplot(u,v,c)    
%         imshow(abs(gaborResult{i,j}),[]);
%     end
% end
% title('Magnitudes of Gabor filters')